@@ -19,14 +19,18 @@
 #ifndef KEEPASSX_EDITENTRYWIDGET_H
 #define KEEPASSX_EDITENTRYWIDGET_H
 
+#include <QButtonGroup>
+#include <QCompleter>
 #include <QModelIndex>
+#include <QPointer>
 #include <QScopedPointer>
 
-#include "gui/EditWidget.h"
 #include "config-keepassx.h"
+#include "gui/EditWidget.h"
 
 class AutoTypeAssociations;
 class AutoTypeAssociationsModel;
+class CustomData;
 class Database;
 class EditWidgetIcons;
 class EditWidgetProperties;
@@ -37,20 +41,25 @@ class EntryHistoryModel;
 class QButtonGroup;
 class QMenu;
 class QSortFilterProxyModel;
-class QStackedLayout;
+class QStringListModel;
 #ifdef WITH_XC_SSHAGENT
 #include "sshagent/KeeAgentSettings.h"
 class OpenSSHKey;
 #endif
+#ifdef WITH_XC_BROWSER
+class EntryURLModel;
+#endif
 
-namespace Ui {
+namespace Ui
+{
     class EditEntryWidgetAdvanced;
     class EditEntryWidgetAutoType;
+    class EditEntryWidgetBrowser;
     class EditEntryWidgetSSHAgent;
     class EditEntryWidgetMain;
     class EditEntryWidgetHistory;
     class EditWidget;
-}
+} // namespace Ui
 
 class EditEntryWidget : public EditWidget
 {
@@ -58,15 +67,13 @@ class EditEntryWidget : public EditWidget
 
 public:
     explicit EditEntryWidget(QWidget* parent = nullptr);
-    ~EditEntryWidget();
+    ~EditEntryWidget() override;
 
-    void loadEntry(Entry* entry, bool create, bool history, const QString& parentName,
-                   Database* database);
+    void
+    loadEntry(Entry* entry, bool create, bool history, const QString& parentName, QSharedPointer<Database> database);
 
-    void createPresetsMenu(QMenu* expirePresetsMenu);
-    QString entryTitle() const;
+    Entry* currentEntry() const;
     void clear();
-    bool hasBeenModified() const;
 
 signals:
     void editFinished(bool accepted);
@@ -78,6 +85,9 @@ private slots:
     void cancel();
     void togglePasswordGeneratorButton(bool checked);
     void setGeneratedPassword(const QString& password);
+#ifdef WITH_XC_NETWORKING
+    void updateFaviconButtonEnable(const QString& url);
+#endif
     void insertAttribute();
     void editCurrentAttribute();
     void removeCurrentAttribute();
@@ -85,6 +95,7 @@ private slots:
     void protectCurrentAttribute(bool state);
     void revealCurrentAttribute();
     void updateAutoTypeEnabled();
+    void openAutotypeHelp();
     void insertAutoTypeAssoc();
     void removeAutoTypeAssoc();
     void loadCurrentAssoc(const QModelIndex& current);
@@ -111,21 +122,33 @@ private slots:
     void decryptPrivateKey();
     void copyPublicKey();
 #endif
+#ifdef WITH_XC_BROWSER
+    void updateBrowserModified();
+    void updateBrowser();
+    void insertURL();
+    void removeCurrentURL();
+    void editCurrentURL();
+    void updateCurrentURL();
+#endif
 
 private:
     void setupMain();
     void setupAdvanced();
     void setupIcon();
     void setupAutoType();
+#ifdef WITH_XC_BROWSER
+    void setupBrowser();
+#endif
 #ifdef WITH_XC_SSHAGENT
     void setupSSHAgent();
 #endif
     void setupProperties();
     void setupHistory();
+    void setupEntryUpdate();
     void setupColorButton(bool foreground, const QColor& color);
 
     bool passwordsEqual();
-    void setForms(const Entry* entry, bool restore = false);
+    void setForms(Entry* entry, bool restore = false);
     QMenu* createPresetsMenu();
     void updateEntryData(Entry* entry) const;
 #ifdef WITH_XC_SSHAGENT
@@ -135,12 +158,11 @@ private:
 
     void displayAttribute(QModelIndex index, bool showProtected);
 
-    Entry* m_entry;
-    Database* m_database;
+    QPointer<Entry> m_entry;
+    QSharedPointer<Database> m_db;
 
     bool m_create;
     bool m_history;
-    bool m_saved;
 #ifdef WITH_XC_SSHAGENT
     bool m_sshAgentEnabled;
     KeeAgentSettings m_sshAgentSettings;
@@ -150,11 +172,21 @@ private:
     const QScopedPointer<Ui::EditEntryWidgetAutoType> m_autoTypeUi;
     const QScopedPointer<Ui::EditEntryWidgetSSHAgent> m_sshAgentUi;
     const QScopedPointer<Ui::EditEntryWidgetHistory> m_historyUi;
+    const QScopedPointer<Ui::EditEntryWidgetBrowser> m_browserUi;
+    const QScopedPointer<CustomData> m_customData;
+
     QWidget* const m_mainWidget;
     QWidget* const m_advancedWidget;
     EditWidgetIcons* const m_iconsWidget;
     QWidget* const m_autoTypeWidget;
+#ifdef WITH_XC_SSHAGENT
     QWidget* const m_sshAgentWidget;
+#endif
+#ifdef WITH_XC_BROWSER
+    bool m_browserSettingsChanged;
+    QWidget* const m_browserWidget;
+    EntryURLModel* const m_additionalURLsDataModel;
+#endif
     EditWidgetProperties* const m_editWidgetProperties;
     QWidget* const m_historyWidget;
     EntryAttributes* const m_entryAttributes;
@@ -166,6 +198,8 @@ private:
     AutoTypeAssociationsModel* const m_autoTypeAssocModel;
     QButtonGroup* const m_autoTypeDefaultSequenceGroup;
     QButtonGroup* const m_autoTypeWindowSequenceGroup;
+    QCompleter* const m_usernameCompleter;
+    QStringListModel* const m_usernameCompleterModel;
 
     Q_DISABLE_COPY(EditEntryWidget)
 };
